/*
*/

#include "zGo.h"
#include "zGoEngine.h"

#include "config.h"
#include "gnugo.h"

#include "liberty.h"
#include "sgftree.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <process.h>
#include <math.h>


extern FILE *zLogFile;

void zScore(zGoData *_goData)
{
	int i,j,k;
	int color;
	int scoreDone = FALSE;
	int boardTmp[MAX_BOARD][MAX_BOARD];

	// LOAD BOARD DATA INTO TMP BUFFER, NEED MODIFY IT
	for (i = 0; i < _goData->board_size; i++) {
		for (j = 0; j < _goData->board_size; j++) {
			boardTmp[i][j] = _goData->board[POS(i,j)];
		}
	}

	removeDeadStone(boardTmp,_goData);

	while (!scoreDone) {
		for (i = 0; i < _goData->board_size; i++) {
			for (j = 0; j < _goData->board_size; j++) {
				color = boardTmp[i][j];
				if (color == 0) {
					check4Neib(i,j,boardTmp,_goData);
				}
			}
		}
		scoreDone = checkScoreDone(boardTmp,_goData);
	}

	finalScore(boardTmp, _goData);
}

void check4Neib(int i, int j, int boardTmp[MAX_BOARD][MAX_BOARD], zGoData* _goData)
{
	int k;
	int neibColor[4];

	for (k=0;k<4;k++) {
		neibColor[k] = 0;
	}
	if (i > 0) {
		neibColor[0] = boardTmp[i-1][j];
	}
	if (i < _goData->board_size-1) {
		neibColor[1] = boardTmp[i+1][j];
	}
	if (j > 0) {
		neibColor[2] = boardTmp[i][j-1];
	}
	if (j < _goData->board_size-1) {
		neibColor[3] = boardTmp[i][j+1];
	}
	for (k=0;k<4;k++) {
		if (neibColor[k] == 1 || neibColor[k] == 11) {
			boardTmp[i][j] = 11;
			break;
		}
		if (neibColor[k] == 2 || neibColor[k] == 12) {
			boardTmp[i][j] = 12;
			break;
		}
	}
}

int checkScoreDone(int boardTmp[MAX_BOARD][MAX_BOARD], zGoData *_goData) {
	int i,j;

	for (i = 0; i < _goData->board_size; i++) {
		for (j = 0; j < _goData->board_size; j++) {
			if (boardTmp[i][j] == 0) return FALSE;
		}
	}
	return TRUE;
}

int finalScore(int boardTmp[MAX_BOARD][MAX_BOARD], zGoData *_goData) {
	int i,j,color;

	int bTerritory = 0;
	int wTerritory = 0;
	for (i = 0; i < _goData->board_size; i++) {
		for (j = 0; j < _goData->board_size; j++) {
			color = boardTmp[i][j];
			if (color == 11) {
				wTerritory++;
			}
			if (color == 12) {
				bTerritory++;
			}
		}
	}
	_goData->bCaptured=black_captured;
	_goData->wCaptured=white_captured;
	_goData->bTerritory=bTerritory;
	_goData->wTerritory=wTerritory;
	_goData->bScore=white_captured+bTerritory;
	_goData->wScore=black_captured+wTerritory;
	return TRUE;
}

int removeDeadStone(int boardTmp[MAX_BOARD][MAX_BOARD], zGoData *_goData) {
	int i,j,color;
	struct dragon_data *dragon1;
	for (i = 0; i < _goData->board_size; i++) {
		for (j = 0; j < _goData->board_size; j++) {
			dragon1 = &(_goData->dragon1[POS(i,j)]);
			color = boardTmp[i][j];
			if (dragon1->status == DEAD) {
				boardTmp[i][j] = EMPTY;
				if (color == BLACK) {
					black_captured++;
				}
				if (color == WHITE) {
					white_captured++;
				}
			}
		}
	}
}

void localGuangzi(int dragonID1, int dragonID2) {
	/*
	char *cmdName="E:\\kevin\\gnugo\\gnugo-3.2\\zGoSubProcess\\Debug\\zGoSubProcess.exe";
	char *args[4];
	args[0] = "child";
	args[1] = "spawn??";
	args[2] = "two";
	args[3] = NULL;
	_spawnv( _P_WAIT, cmdName, args);
	*/
	zPosition closestStone1;
	zPosition closestStone2;
	locateClosestStone(dragonID1,dragonID2,&closestStone1);
	locateClosestStone(dragonID2,dragonID1,&closestStone2);
	fprintf(zLogFile,"closestStone, x=%i y=%i\n", closestStone1.x,closestStone1.y);
	fprintf(zLogFile,"closestStone, x=%i y=%i\n", closestStone2.x,closestStone2.y);
}

// LOCATE CLOSEST STONE INSIDE DRAGON1 TO DRAGON2
void locateClosestStone(int dragonID1, int dragonID2, zPosition *closestStone)
{	
	int i,j;
	struct dragon_data *dragon1;
	struct dragon_data *dragon2;
	zPosition posA1[360];
	zPosition posA2[360];
	double dist;
	double minDist = 99999999;

	getPosForDragon(dragonID1,posA1);
	dragon1 = &dragon[POS(posA1[0].x, posA1[0].y)];

	getPosForDragon(dragonID2,posA2);
	dragon2 = &dragon[POS(posA2[0].x, posA2[0].y)];

	for (i=0;i<dragon1->size;i++) {
		//fprintf(zLogFile,"i=%i x=%i y=%i\n", i,posA1[i].x,posA1[i].y);
		for (j=0;j<dragon2->size;j++) {
			dist = straightDistance(&posA1[i],&posA2[j]);
			if (minDist > dist) {
				minDist = dist;
				closestStone->x = posA1[i].x;
				closestStone->y = posA1[i].y;
			}
		}
	}
}

double straightDistance(zPosition *_p1, zPosition *_p2)
{
	return sqrt((_p1->x-_p2->x)*(_p1->x-_p2->x) +
			    (_p1->y-_p2->y)*(_p1->y-_p2->y));
}

// CAL BLOCK DIST FROM POS1 TO POS2, STORE RESULT IN DIST
void blockDistance(zPosition pos1, zPosition pos2, zPosition dist)
{
	dist.x = abs(pos1.x - pos2.x);
	dist.y = abs(pos1.y - pos2.y);
}

void getPosForDragon(int _dragonID, zPosition posA[])
{
	int i,j;
	int posCount = 0;
	//printf("zGetPosForDragon 1\n");
	for (i = 0; i < board_size; i++) {
		for (j = 0; j < board_size; j++) {
			if (dragon[POS(i,j)].id == _dragonID) {
				//printf("i,j,posCount=%i, %i, %i\n", i,j,posCount);
				posA[posCount].x = i;
				posA[posCount].y = j;
				posCount++;
			}
		}
	}
	//printf("zGetPosForDragon 9\n");
}


